/***************************************************************************
 *   Copyright (C) 2009 by Mirko Lindner,,,   *
 *   vegyraupe@mira   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <gtkmm.h>
#include <zim/file.h>
#include <zim/fileiterator.h>
#include <zim/search.h>
#include <zim/articlesearch.h>
#include <iostream>
#include <string>

#include <fstream>

// #include "main_window.hh"
#include "search_dialog.hh"

extern "C" {
   #include "gtkhtml/gtkhtml.h"
 }

using namespace Gtk;
Glib::RefPtr<Gtk::TextBuffer> buffer;
unsigned int seed = static_cast<unsigned int>(time(0));
zim::Search::Results result;
std::string res;
std::string content;
std::string file;
zim::File f;
Window *window;

Widget *html;
GtkWidget *html_wg;

static zim::File zimFile;
static bool initialized = false;

// // // misc functions

const zim::File get_file()
{
  if (!initialized){
    std::cout<<"file not initialized :"<<file<<".\n";
    zim::File zimFile(file);
    initialized = true;
  }else{
    std::cout<<"file initialized :"<<file<<".\n";
  }
  std::cout<<"returning file"<<".\n";
  return zimFile;
}

// // copy article html from while loop into global variable
void set_article(std::string txt)
{
    content = txt;
}

// // // externally called functions

// // display random article
void show_random(){

    std::cout<<"random called"<<".\n";
    zim::File m = get_file();
    std::cout<<"random got file"<<".\n";
  
    while(true){
        std::cout<<"random running loop"<<".\n";
        zim::size_type idx = static_cast<zim::size_type>(static_cast<double>(m.getCountArticles()) * rand_r(&seed) / RAND_MAX);

        zim::Article article = m.getArticle(idx);

        std::string res = article.getPage(false, 10);
        if (res.size() > 0){
            set_article(res);
            break;
        }
    }
    gtk_html_flush(GTK_HTML(html_wg));
    gtk_html_load_from_string(GTK_HTML(html_wg), content.c_str(), -1);
}

// // display search dialog
void search_window(main_window *window_x){

    search_dialog(window_x, " ");

}

// // // meta functions

// // set displayed html to given url
// // FIXME: returns several articles and displays on one page ... why?
void set_html(const gchar *url){

// //     create ZIM file accessor
    zim::File m = get_file();

// //     convert url to string
    std::string term;
    term.assign( url );

// //     replace '+' signs with spaces in url
    std::replace(term.begin(), term.end(), '+', ' ');
    
// //     find and declare namespace
    size_t found;
    size_t found2;
    char *ns;

    found=term.find("/");
    if (found!=std::string::npos){
        if (found != (int) 0){
            ns = &term[0];
            term = term.substr(found + 1, strlen(url));
        }else{
            term = term.substr(1, strlen(url));
            ns = &term[0];
            found2 = term.find("/");
            term = term.substr(found2 + 1, strlen(url));
        }
    }else{
        ns = "A";
    }

// //     try to retrieve article
    try {
        zim::File::const_iterator iterator = m.find(ns[0], zim::QUnicodeString(term));
        zim::Article article = m.getArticle(iterator.getIndex());

// //         get redirect article if needed
        if (article.isRedirect())
        {
            article = article.getRedirectArticle();
        }

// //         get content html
        content = article.getData().data();
        
// //         get article length
        int length = article.getArticleSize();
        
// //         limit html output to article length
        std::string str2 = content.substr (0,length);

// //         emtpy gtkhtml widget
        gtk_html_flush(GTK_HTML(html_wg));

// //         load new data into gtkhtml widget
        gtk_html_load_from_string(GTK_HTML(html_wg), str2.c_str(), -1);
        std::cout<<"term:"<<term<<".\n";
        std::cout<<"url:"<<url<<".\n";
    }catch(...){
    }
}


// // test functions
void set_search(const gchar *url){
//     std::cout<<"url"<<url<<".";
    zim::File z = get_file();
    
    zim::Search::Results result;
    result.clear();
    zim::Search search(z);
    std::string term;
    term.assign( url );
    std::replace(term.begin(), term.end(), '+', ' ');
    search.search(result, term);
    if(result.size() == 0) {
        
    }
    else if (result.size() == 1){

        zim::Article article = z.getArticle(result[0].getArticle().getIndex());
        
        std::string content = article.getPage(false, 10);
        
        gtk_html_flush(GTK_HTML(html_wg));
        gtk_html_load_from_string(GTK_HTML(html_wg), content.c_str(), -1);
    }else{
        std::string res;
        for (unsigned i = 0; i < result.size(); ++i) {
          res += "<li><a href=" + result[i].getArticle().getUrl().toXML() + ">" + result[i].getArticle().getTitle().toXML() + "</a></li>";
        }
        
        gtk_html_flush(GTK_HTML(html_wg));
        gtk_html_load_from_string(GTK_HTML(html_wg), res.c_str(), -1);
    
    }
//     char* ns = "A";
    
//     try {
//     zim::File::const_iterator iterator = z.find(ns[0], zim::QUnicodeString(term));
//     zim::Article article = z.getArticle(iterator.getIndex());
//     if (article.isRedirect())
//       {
//           article = article.getRedirectArticle();
//       }
//     content = article.getData().data();
//     gtk_html_flush(GTK_HTML(html_wg));
//     gtk_html_load_from_string(GTK_HTML(html_wg), content.c_str(), -1);
//     }catch(...){
//     }

    std::cout<<url<<".\n";
}

void articles2file(){
    
    std::ofstream myfile;
    myfile.open ("articles.txt");
    zim::File f = get_file();
    for (zim::File::const_iterator it = f.begin(); it != f.end(); ++it)
        {
            myfile << it->getUrl() << '\n';
        }
    myfile.close();
    
    }



// // // window response functions

// // open requested link
// FIXME show error window when dead-link requested
bool on_link_clicked(GtkHTML *html, const gchar *url)
{

    set_html(url);
//     set_berlin(url);
    return true;
}


// // main function
int main(int argc, char **argv)
{
    if (argc == 2)
    {
      file = argv[1];

      std::cout<<"vido is here file:"<<file<<".\n";
      
      Main kit(argc, argv);

      main_window window;
      window.set_title("Vido");
  //     window.set_border_width(0);
      window.set_default_size(320, 240);
  //     window.set_resizable(0);

      html_wg = gtk_html_new();
      
      html = Glib::wrap(html_wg);
      g_signal_connect( G_OBJECT( html_wg ), "link_clicked", G_CALLBACK( on_link_clicked ), NULL );
      gtk_html_set_caret_mode(GTK_HTML(html_wg),true);

      
      ScrolledWindow scrolled_window;
      
      scrolled_window.add(*html);
      scrolled_window.set_policy(POLICY_NEVER, POLICY_AUTOMATIC);
      
      window.add(scrolled_window);
      window.show_all();
      
      show_random();
  //     articles2file();
  //     set_berlin();
      Main::run(window);
    }else{
      std::cout<<"You provided too many or too few arguments("<< argc <<").\n";
      std::cout<<"Vido is to be used as follows:"<<".\n";
      std::cout<<"vido [PATH TO ZIM FILE]"<<".\n";
    }
    return 0;

}
